@@ -229,7 +229,11 @@ int dap_maxex2 = DAP_MAXEX2;
 #else
 int dap_maxex2 = 20;          /* max number of values for exact test */
 #endif
-
+#ifdef DAP_CATTOL
+double dap_cattol = DAP_CATTOL;
+#else
+double dap_cattol = 0.0000005; /* tolerance for convergence in categ() */
+#endif
 
 /* Parameters for memory files */
 #ifdef DAP_NRFILES
@@ -324,14 +328,15 @@ void lsmeans(char fname[], char method[], double alpha, char varlist[], char tre
                 char marks[], char format[]);
 void linreg(char fname[], char ylist[], char x0list[], char x1list[],
 				char marks[], char xname[], double level);
-pict *plotlinreg(char *fname, char *ylist, char *x1list, char *marks, int nmarks, double level);
+pict *plotlinreg(char *fname, char *ylist, char *x1list, char *style,
+		char *marks, int nmarks, double level);
 void logreg(char fname[], char yspec[], char x0list[], char x1list[],
 				char marks[], char xname[], double level);
-pict *plotlogreg(char *fname, char *yspec, char *x1list, int ngroups,
+pict *plotlogreg(char *fname, char *yspec, char *x1list, char *style, int ngroups,
 			char *marks, int nmarks, double level);
 void nonparam(char fname[], char variables[], char marks[]);
 void categ(char *dataset, char *varlist, double (*prob)(),
-			double param[], char *select, char *partvars);
+		double param[], char *select, char *partvars, char *trace);
 
 double varnorm();
 double varunif();
