@@ -494,7 +494,7 @@ if (vname)
 return -1;
 }
 
-static int dap_arrnum(char vname[], int *dim)
+int dap_arrnum(char vname[], int *dim)
 {
 int v;
 int n;
@@ -803,7 +803,7 @@ if ((nread = getline(dap_in[dap_ono], line)) <= 0)
 	{
 	eof[dap_ono] = 1;
 	dap_obs[dap_ono].do_valid = 0;
-	fprintf(stderr, "(step) %d lines read from  %s\n",
+	fprintf(stderr, "(step) %d lines read from %s\n",
 		lineno[dap_ono] - 1, dap_in[dap_ono]->dfile_name);
 	if (!nread)
 		fputs("(step) WARNING: terminated on null line\n", stderr);
@@ -1156,7 +1156,7 @@ int c;
 int v;
 int onum;
 int maxnamlen;			/* max length of variable name in dataset */
-static dataobs baseobs;
+static dataobs baseobs;		/* dataobs for append */
 char *varspec;
 char *oldvmem;
 char *newvmem;
@@ -1170,18 +1170,18 @@ static char dimstr[7];	/* depends on dap_maxvar <= 10,000 */
 int *clearvar;		/* indices of variables to clear in APPEND */
 int nclear;		/* number of variables to clear */
 
-if (!datsetinit)
+if (!datsetinit)	/* need to allocate and initiate baseobs? */
 	{
 	datsetinit = 1;
 	initdo(&baseobs);
 	}
-if (dap_in[0])
-	{
+if (dap_in[0])		/* if input file left hanging... */
+	{		/* ... finish it up */
 	dfclose(dap_in[0]);
 	dap_in[0] = (DFILE *) NULL;
 	}
-if (dap_out)
-	{
+if (dap_out)		/* if output file left hanging... */
+	{		/* ... finish it up */
 	dfclose(dap_out);
 	dap_out = (DFILE *) NULL;
 	}
@@ -1196,7 +1196,7 @@ for (v = 0; v < dap_maxvar; v++)
 	newvar[v] = newvmem + v * (dap_namelen + 1);
 dold = dap_malloc(strlen(oldname) + strlen(dap_setdir) + 2, oldname);
 dnew = dap_malloc(strlen(newname) + strlen(dap_setdir) + 2, newname);
-dap_name(dold, oldname);
+dap_name(dold, oldname);	/* set up dataset names */
 dap_name(dnew, newname);
 clearvar = NULL;
 if (!strcmp(action, "RENAME"))
@@ -1371,8 +1371,8 @@ else if (!strcmp(action, "REMOVE"))
 	}
 else if (!strcmp(action, "APPEND"))
 	{
-	if (!(dap_out = dfopen(newname, "r")))
-		{
+	if (!(dap_out = dfopen(newname, "r")))	/* no dataset to append to... */
+		{				/* ...so just copy. */
 		if (!(dap_out = dfopen(newname, "w")))
 			{
 			fprintf(dap_err,
@@ -1383,7 +1383,7 @@ else if (!strcmp(action, "APPEND"))
 		if (!(dap_in[0] = dfopen(oldname, "r")))
 			{
 			fprintf(dap_err,
-				"(dataset) Can't read old data set for append: %s\n",
+				"(dataset) can't read old data set for append: %s\n",
 				oldname);
 			exit(1);
 			}
@@ -1395,7 +1395,7 @@ else if (!strcmp(action, "APPEND"))
 		dap_out = (DFILE *) NULL;
 		return;
 		}
-	else
+	else				/* really append */
 		{
 		inset(newname);			/* to get variable list */
 		outset("dap_null", "");		/* so don't output anywhere */
@@ -1409,7 +1409,7 @@ else if (!strcmp(action, "APPEND"))
 			}
 		dfclose(dap_out);
 		dap_out = (DFILE *) NULL;
-		inset(oldname);		/* open dataset to append to newname */
+		inset(oldname);		/* open "old" dataset to append to newname */
 		dap_obs[0].do_ovar = 0;
 		for (v = 0, maxnamlen = 0; v < baseobs.do_nvar; v++)
 			{		/* find longest name length in newname */
@@ -1424,14 +1424,25 @@ else if (!strcmp(action, "APPEND"))
 		for (v = 0, nclear = 0; v < baseobs.do_nvar; v++)
 			{
 			if ((onum = dap_varnum(baseobs.do_nam[v])) < 0)
-				{	/* if not in oldname */
-				/* give it newname's specs for it */
+				{	/* get index in oldname */
+				/* if none give it newname's specs for it */
 				sprintf(varspec, "%s %d", baseobs.do_nam[v],
 							baseobs.do_len[v]);
 				onum = dap_vd(varspec, 0);
 				/* mark for setting to "", 0, and 0.0 */
 				clearvar[nclear++] = onum;
 				}
+			else		/* if it's there, check type, len */
+				{
+				if (dap_obs[0].do_len[onum] != baseobs.do_len[v])
+					{
+					fprintf(stderr,
+		"dataset: variable %s has different lengths (%d appended to %d) in datasets\n",
+					baseobs.do_nam[v], dap_obs[0].do_len[onum],
+					baseobs.do_len[v]);
+					exit(1);
+					}
+				}
 			dap_obs[0].do_out[dap_obs[0].do_ovar++] = onum;
 			}
 		if (!(dap_out = dfopen(newname, "a")))
