@@ -22,6 +22,7 @@
 
 extern dataobs dap_obs[];
 extern FILE *dap_lst;
+extern FILE *dap_log;
 extern FILE *dap_err;
 extern char *dap_dapname;
 
@@ -623,7 +624,6 @@ int s;
 int i;
 int v;
 char *varname;
-char *ctype;
 int n;
 int nvar;
 int number;
@@ -631,10 +631,9 @@ int number;
 if (!varspec)
 	return 0;
 varname = dap_malloc(dap_namelen + 1, "");
-ctype = dap_malloc(dap_namelen + 1, "");
 for (s = 0; varspec[s] == ' '; s++)
 	;
-for (nvar = 0, number = 0; varspec[s]; nvar++)
+for (nvar = 0, number = 0; varspec[s]; )
 	{
 	classtype[nvar] = 0;
 	for (i = 0; varspec[s + i] && varspec[s + i] != ' '; i++)
@@ -656,60 +655,9 @@ for (nvar = 0, number = 0; varspec[s]; nvar++)
 		varv[nvar] = v;
 		while (varspec[s] == ' ')
 			s++;
-		if ('0' <= varspec[s] && varspec[s] <= '9')
+		if (number)
 			{
-			if (dap_obs[0].do_len[v] == DBL)
-				{
-				for (n = 0, i = 0;
-					'0' <= varspec[s + i] && varspec[s + i] <= '9';
-						i++)
-					n = 10 * n + varspec[s + i] - '0';
-				if (!n || varspec[s + i] != ' ')
-					{
-					fprintf(dap_err,
-				"(groupparse) invalid number of groups: %s\n",
-						varspec + s);
-					exit(1);
-					}
-				s += i;
-				while (varspec[s] == ' ')
-					s++;
-				for (i = 0; varspec[s + i] && varspec[s + i] != ' '; i++)
-					{
-					if (i < dap_namelen)
-						ctype[i] = varspec[s + i];
-					else
-						{
-						ctype[i] = '\0';
-						fprintf(dap_err,
-						"(groupparse) class type too long: %s\n",
-								ctype);
-						exit(1);
-						}
-					}
-				ctype[i] = '\0';
-				s += i;
-				while (varspec[s] == ' ')
-					s++;
-				if (!strcmp(ctype, "WIDTH"))
-					classtype[nvar] = -n;
-				else if (!strcmp(ctype, "COUNT"))
-					classtype[nvar] = n;
-				else
-					{
-					fprintf(dap_err,
-						"(groupparse) invalid class type: %s\n",
-						ctype);
-					exit(1);
-					}
-				if (n > dap_maxbars)
-					{
-					fprintf(dap_err,
-					"(groupparse) too many classes: %d\n", n);
-					exit(1);
-					}
-				}
-			else
+			if (dap_obs[0].do_len[v] != DBL)
 				{
 				fprintf(dap_err,
 				"(groupparse) grouping variable must be of type double: %s\n",
@@ -719,20 +667,63 @@ for (nvar = 0, number = 0; varspec[s]; nvar++)
 			}
 		else
 			{
-			fprintf(dap_err, "(groupparse) missing number of groups for %s\n",
-					varname);
-			exit(1);
+			if ('0' <= varspec[s] && varspec[s] <= '9')
+				{
+				if (dap_obs[0].do_len[v] == DBL)
+					{
+					for (n = 0, i = 0;
+						'0' <= varspec[s + i] && varspec[s + i] <= '9';
+							i++)
+						n = 10 * n + varspec[s + i] - '0';
+					if (!n || (varspec[s + i] != '#' &&
+							varspec[s + i] != '^'))
+						{
+						fprintf(dap_err,
+					"(groupparse) invalid number of groups: %s\n",
+							varspec + s);
+						exit(1);
+						}
+					switch (varspec[s + i])
+						{
+					case '^':
+						classtype[nvar] = -n;
+						break;
+					case '#':
+						classtype[nvar] = n;
+						break;
+					default:
+						fprintf(dap_err,
+							"(groupparse) invalid class type: %s\n",
+							varspec + s + i);
+						exit(1);
+						}
+					s += i + 1;
+					while (varspec[s] == ' ')
+						s++;
+					if (n > dap_maxbars)
+						{
+						fprintf(dap_err,
+						"(groupparse) too many classes: %d\n", n);
+						exit(1);
+						}
+					}
+				else
+					{
+					fprintf(dap_err,
+				"(groupparse) grouping variable must be of type double: %s\n",
+						varname);
+					exit(1);
+					}
+				}
+			else
+				{
+				fprintf(dap_err,
+				"(groupparse) missing number of groups for %s\n",
+						varname);
+				exit(1);
+				}
 			}
-		}
-	else if (!nvar)
-		{
-		if (!strcmp(varname, "NUMBER"))
-			number = GRPNUMBER;
-		else if (!strcmp(varname, "FRACTION"))
-			number = GRPFRACTION;
-		else if (!strcmp(varname, "PERCENT"))
-			number = GRPPERCENT;
-		classtype[0] = 0;
+		nvar++;
 		}
 	else if (number < 0)
 		{
@@ -741,6 +732,17 @@ for (nvar = 0, number = 0; varspec[s]; nvar++)
 		if (varname[1] == '0')
 			classtype[0] |= GRP0;
 		}
+	else if (!nvar)
+		{
+		if (!strcmp(varname, "#"))
+			number = GRPNUMBER;
+		else if (!strcmp(varname, "/"))
+			number = GRPFRACTION;
+		else if (!strcmp(varname, "%"))
+			number = GRPPERCENT;
+		classtype[0] = 0;
+		nvar++;
+		}
 	else
 		{
 		fprintf(dap_err, "(groupparse) unknown variable: %s\n", varname);
@@ -750,9 +752,8 @@ for (nvar = 0, number = 0; varspec[s]; nvar++)
 		s++;
 	}
 dap_free(varname);
-dap_free(ctype);
 if (number < 0)
-	return number;
+	return -4 * nvar + number;
 return nvar;
 }
 
@@ -809,10 +810,13 @@ int *ctype;
 double *nummem;
 double **numval;
 int nvar;
+int number;
 int v;
 char *grpname;
 int *grpv;
 int nobs;
+int nnan;
+int allgood;		/* all vars selected are finite */
 double *ptmem;
 double **point;
 double dnobs;
@@ -835,8 +839,15 @@ ctype = (int *) dap_malloc(sizeof(int) * dap_maxvar, "");
 inset(fname);
 nmark = dap_list(marks, markv, dap_maxvar);
 nvar = groupparse(varspec, varv, ctype);
-grpv = (int *) dap_malloc(sizeof(int) * (nvar ? nvar : 1), "");
-if (nvar > 0)
+if (nvar < 0)
+	{
+	number = -((-nvar) % 4);
+	nvar = -(nvar - number) / 4;
+	}
+else
+	number = 0;
+grpv = (int *) dap_malloc(sizeof(int) * (number ? 1 : nvar), "");
+if (!number)
 	{
 	strcpy(grpname, "_");
 	for (v = 0; v < nvar; v++)
@@ -848,11 +859,14 @@ if (nvar > 0)
 		}
 	}
 else
+	{
 	grpv[0] = dap_vd("_N_ -1", 0);
+	varv[0] = grpv[0];
+	}
 outset(outname, "");
 nummem = NULL;
 numval = NULL;
-if (nvar > 0)
+if (!number)
 	{
 	nummem = (double *) dap_malloc(sizeof(double) * nvar * dap_maxval, "");
 	numval = (double **) dap_malloc(nvar * dap_maxval * sizeof(double *), "");
@@ -864,17 +878,20 @@ if (nvar > 0)
 		point[v] = ptmem + v * (dap_maxbars + 1);
 		}
 	}
-for (dap_mark(), nobs = 0, more = 1; more; nobs++)
+for (dap_mark(), nobs = 0, nnan = 0, more = 1; more; )
 	{
 	more = step();
 	if (dap_newpart(markv, nmark))
 		{
-		getpoints(numval, nvar, ctype, nobs, point);
+		if (!number)
+			getpoints(numval, nvar, ctype, nobs, point);
 		dnobs = (double) nobs;
 		dap_rewind();
 		count = 1.0;
 		countinc = 1.0;
-		if (nvar < 0)
+		if (nnan > 0)
+			fprintf(dap_log, "(group) %d NaNs\n", nnan);
+		if (number)
 			{
 			if (ctype[0] & GRP0)
 				count = 0.0;
@@ -884,50 +901,100 @@ for (dap_mark(), nobs = 0, more = 1; more; nobs++)
 				count = dnobs - 1.0 + count;
 				}
 			}
-		for ( ; step() && !dap_newpart(markv, nmark); count += countinc)
+		for ( ; step() && !dap_newpart(markv, nmark); )
 			{
-			switch (nvar)
+			if (number)
 				{
-			case GRPNUMBER:
-				dap_obs[0].do_dbl[grpv[0]] = count;
-				break;
-			case GRPFRACTION:
-				dap_obs[0].do_dbl[grpv[0]] = count / dnobs;
-				break;
-			case GRPPERCENT:
-				dap_obs[0].do_dbl[grpv[0]] = 100.0 * count / dnobs;
-				break;
-			default:
-				for (v = 0; v < nvar; v++)
+				for (v = 1, allgood = 1; v < nvar; v++)
 					{
-					for (p = 1;
-						dap_obs[0].do_dbl[varv[v]] >
-							point[v][p]; p++)
-						;
-					dap_obs[0].do_dbl[grpv[v]] = (double) p;
+					if (!finite(dap_obs[0].do_dbl[varv[v]]))
+						{
+						allgood = 0;
+						break;
+						}
 					}
-				break;
+				if (allgood)
+					{
+					switch (number)
+						{
+					case GRPNUMBER:
+						dap_obs[0].do_dbl[grpv[0]] = count;
+						break;
+					case GRPFRACTION:
+						dap_obs[0].do_dbl[grpv[0]] = count / dnobs;
+						break;
+					case GRPPERCENT:
+						dap_obs[0].do_dbl[grpv[0]] = 100.0 * count / dnobs;
+						break;
+						}
+					count += countinc;
+					}
+				else
+					dap_obs[0].do_dbl[grpv[0]] = 0.0 / 0.0;
+				}
+			else
+				{
+				for (v = 0, allgood = 1; v < nvar; v++)
+					{
+					if (!finite(dap_obs[0].do_dbl[varv[v]]))
+						{
+						allgood = 0;
+						break;
+						}
+					}
+				if (allgood)
+					{
+					for (v = 0; v < nvar; v++)
+						{
+						for (p = 1;
+							dap_obs[0].do_dbl[varv[v]] >
+								point[v][p]; p++)
+							;
+						dap_obs[0].do_dbl[grpv[v]] = (double) p;
+						}
+					}
+				else
+					for (v = 0; v < nvar; v++)
+						dap_obs[0].do_dbl[grpv[v]] = 0.0 / 0.0;
 				}
 			output();
 			dap_mark();
 			}
 		nobs = 0;
+		nnan = 0;
 		}
-	if (nvar > 0)
+	if (number)
+		{
+		for (v = 1, allgood = 1; v < nvar; v++)
+			{
+			if (!finite(dap_obs[0].do_dbl[varv[v]]))
+				{
+				allgood = 0;
+				break;
+				}
+			}
+		if (allgood)
+			nobs++;
+		else
+			nnan++;
+		}
+	else
 		{
 		if (nobs < dap_maxval)
 			{
-			for (v = 0; v < nvar; v++)
+			for (v = 0, allgood = 1; v < nvar; v++)
 				{
-				numval[v][nobs] = dap_obs[0].do_dbl[varv[v]];
-				if (!finite(numval[v][nobs]))
+				if (!finite(dap_obs[0].do_dbl[varv[v]]))
 					{
-					fprintf(dap_err,
-						"(group) NaN value %d for %s\n",
-						nobs, dap_obs[0].do_nam[varv[v]]);
-					exit(1);
+					allgood = 0;
+					break;
 					}
+				numval[v][nobs] = dap_obs[0].do_dbl[varv[v]];
 				}
+			if (allgood)
+				nobs++;
+			else
+				nnan++;
 			}
 		else
 			{
@@ -936,7 +1003,7 @@ for (dap_mark(), nobs = 0, more = 1; more; nobs++)
 			}
 		}
 	}
-if (nvar > 0)
+if (!number)
 	{
 	dap_free(nummem);
 	dap_free(numval);
