@@ -33,20 +33,29 @@ static void plot1(double x[], double y[], int nobs,
 	pict p[], pict a[], int pn, char style[],
 	double (*xfunct)(), double (*yfunct)(), int doaxes)
 {
+int titlelen;
 static char *title0 = NULL;
 char *title1;
 int overlay;
 int n;
 int v;
 int s;
-char axspec[3];
+char *axspec;
 int t;
 double pictr;
 
 dap_swap();
+if (dap_title)
+	titlelen = strlen(dap_title) + 1;
+else
+	titlelen = 0;
 if (!title0)
-	title0 = dap_malloc(dap_linelen + 1, "");
-title1 = dap_malloc(dap_linelen + 1, "");
+	{
+	title0 = dap_malloc(dap_linelen + titlelen + 1, "dap_linelen");
+	if (dap_title)
+		strcpy(title0, dap_title);
+	}
+title1 = dap_malloc(dap_linelen + titlelen + 1, "dap_linelen");
 for (s = 0; style[s] == ' '; s++)
 	;
 overlay = 0;
@@ -61,8 +70,8 @@ if (style[s] == 'o')
 	}
 while (style[s] == ' ')
 	s++;
-strncpy(axspec, style + s, 2);
-axspec[2] = '\0';
+axspec = dap_malloc(3 + strlen(style), "");
+strcpy(axspec, style + s);
 if (overlay == -1)
 	{
 	if (pn)
@@ -137,7 +146,13 @@ if (doaxes)
 					axspec, xfunct, yfunct, title1, PORTRAIT);
 	}
 dap_swap();
+if (doaxes)
+	{
+	dap_free(title0);
+	title0 = NULL;
+	}
 dap_free(title1);
+dap_free(axspec);
 }
 
 static void plotparse(char *xyvar, char *xyname, char *xname, char *yname)
@@ -548,6 +563,7 @@ static void normal1(double x[], double y[], int nobs,
         char varname[], char varlabel[], int markv[], int nmark,
 	pict *p, pict *l, pict *a, int pn)
 {
+int titlelen;
 int r;
 double dr;
 double dnobsp25;
@@ -561,10 +577,17 @@ double *swa;
 int k;
 double w;
 double prob;
-static char caption[46];
+int v;
+char *caption;
 int (*cmp)();
 
+dap_swap();
 cmp = &dblcmp;
+if (dap_title)
+	titlelen = strlen(dap_title) + 1;
+else
+	titlelen = 0;
+caption = dap_malloc(dap_linelen + titlelen + 47, "dap_linelen");
 qsort((void *) y, (size_t) nobs, (size_t) sizeof(double), cmp);
 dnobsp25 = ((double) nobs) + 0.25;
 for (r = 0, sum = 0.0, ss = 0.0, minx = 0.0, maxx = 0.0; r < nobs; r++)
@@ -603,6 +626,26 @@ if (6 < nobs && nobs <= 500)
 	}
 else
 	strcpy(caption, "q-q plot");
+if (dap_title)
+	{
+	strcat(caption, " ");
+	strcat(caption, dap_title);
+	}
+if (nmark)
+	{
+	for (v = 0; v < nmark; v++)
+		{
+		strcat(caption, " ");
+		if (dap_obs[0].do_len[markv[v]] == DBL)
+			sprintf(caption + strlen(caption), "%g",
+						dap_obs[0].do_dbl[markv[v]]);
+		else if (dap_obs[0].do_len[markv[v]] == INT)
+			sprintf(caption + strlen(caption), "%d",
+						dap_obs[0].do_int[markv[v]]);
+		else
+			strcat(caption, dap_obs[0].do_str[markv[v]]);
+		}
+	}
 pict_initpict(NULL, p + pn);
 pict_initpict(p + pn, l + pn);
 pict_initpict(l + pn, a + pn);
@@ -613,6 +656,8 @@ sum /= (double) nobs;
 strcpy(l[pn].pict_type, "LINE");
 pict_line(l + pn, minx, sd * minx + sum, maxx, sd * maxx + sum);
 p[pn].pict_r = 0.05 * pict_autoaxes(p + pn, "z", varlabel, "00", NULL, NULL, caption, PORTRAIT);
+free(caption);
+dap_swap();
 }
 
 pict *normal(char *fname, char *variable, char *marks, int nplots)
@@ -755,11 +800,13 @@ static void histo1(double x[], double xw[][2], int nobs, int varv[], int nbars,
 	char xname[], char *style, double (*xfunct)(),
 	int markv[], int nmark, pict *p, pict *a, int pn)
 {
+int titlelen;
 char *caption;
 int s;
 int w;
 char *word;
-char axspec[3];
+char bordspec[3];
+char axspec[16];	/* you can break this if you really want to */
 double *h;
 double *part;
 int equal;
@@ -775,10 +822,15 @@ int b;
 int xn;
 int xnm1;
 int v;
+double maxy;
 int (*cmp)();
 
 cmp = &dblcmp;
-caption = dap_malloc(dap_linelen + 1, "");
+if (dap_title)
+	titlelen = strlen(dap_title) + 1;
+else
+	titlelen = 0;
+caption = dap_malloc(titlelen + dap_linelen + 1, "");
 word = dap_malloc(dap_namelen + 1, "");
 h = (double *) dap_malloc(sizeof(double) * dap_maxbars, "");
 part = (double *) dap_malloc(sizeof(double) * (dap_maxbars + 1), "");
@@ -791,7 +843,7 @@ equal = 1;
 height = UNSPEC;
 whole = 0;
 htitle[0] = '\0';
-axspec[0] = '\0';
+bordspec[0] = '\0';
 if (x)
 	{
 	qsort((void *) x, (size_t) nobs, (size_t) sizeof(double), cmp);
@@ -804,6 +856,7 @@ else
 	part[0] = xw[0][0];
 	part[nbars] = xw[nobs - 1][0];
 	}
+maxy = 0.0;	/* mark as not used */
 if (style)
 	{
 	for (s = 0; style[s] == ' '; s++)
@@ -834,30 +887,40 @@ if (style)
 			height = FRACTION;
 		else if (!strcmp(word, "ROUND"))
 			whole = 1;
-		else if (!strncmp(word, "MIN", 3))
+		else if (!strncmp(word, "MINX", 4))
 			{
-			if (sscanf(word + 3, "%lf", &part[0]) != 1)
+			if (sscanf(word + 4, "%lf", &part[0]) != 1)
 				{
 				fprintf(dap_err,
-					"(histo1) bad MIN specification: %s\n",
+					"(histo1) bad MINX specification: %s\n",
 					word);
 				exit(1);
 				}
 			}
-		else if (!strncmp(word, "MAX", 3))
+		else if (!strncmp(word, "MAXX", 4))
 			{
-			if (sscanf(word + 3, "%lf", &part[nbars]) != 1)
+			if (sscanf(word + 4, "%lf", &part[nbars]) != 1)
 				{
 				fprintf(dap_err,
-					"(histo1) bad MAX specification: %s\n",
+					"(histo1) bad MAXX specification: %s\n",
+					word);
+				exit(1);
+				}
+			}
+		else if (!strncmp(word, "MAXY", 4))
+			{
+			if (sscanf(word + 4, "%lf", &maxy) != 1)
+				{
+				fprintf(dap_err,
+					"(histo1) bad MAXY specification: %s\n",
 					word);
 				exit(1);
 				}
 			}
 		else if (index("-+0nb", word[0]))
 			{
-			strncpy(axspec, word, 2);
-			axspec[2] = '\0';
+			strncpy(bordspec, word, 2);
+			bordspec[2] = '\0';
 			}
 		else
 			{
@@ -868,6 +931,10 @@ if (style)
 			s++;
 		}
 	}
+if (maxy > 0.0)
+	sprintf(axspec, "%s MAXY%g", bordspec, maxy);
+else
+	strcpy(axspec, bordspec);
 if (equal)
 	{
 	switch (height)
@@ -983,13 +1050,11 @@ for (b = 0; b < nbars; b++)
 caption[0] = '\0';
 if (dap_title)
 	strcpy(caption, dap_title);
-if (dap_title)
+if (nmark)
 	{
 	for (v = 0; v < nmark; v++)
 		{
-		if (caption[0] && !v)
-			strcat(caption, ": ");
-		else if (v)
+		if (caption[0] || v)
 			strcat(caption, " ");
 		if (dap_obs[0].do_len[markv[v]] == DBL)
 			sprintf(caption + strlen(caption), "%g",
@@ -1148,7 +1213,7 @@ dap_free(xname);
 return p;
 }
 
-pict *plotlinreg(char *fname, char *ylist, char *x1list,
+pict *plotlinreg(char *fname, char *ylist, char *x1list, char *style,
 			char *marks, int nmarks, double level)
 {
 int varv[1];
@@ -1160,6 +1225,7 @@ char *plotvars;
 char *plotmarks;
 pict *p;
 int pn;
+char *plotstyle;
 
 inset(fname);
 dap_list(ylist, varv, 1);	/* check that it's only 1 variable */
@@ -1188,7 +1254,10 @@ strcat(plotvars, ylist);
 plotmarks = dap_malloc(strlen(marks) + strlen("_type_") + 2, "");
 strcpy(plotmarks, marks);
 strcat(plotmarks, " _type_");
-p = plot(srtname, plotvars, plotmarks, "o4bb", NULL, NULL, 4 * nmarks);
+plotstyle = dap_malloc(strlen(style) + 4, "");
+strcpy(plotstyle, "o4 ");
+strcat(plotstyle, style);
+p = plot(srtname, plotvars, plotmarks, plotstyle, NULL, NULL, 4 * nmarks);
 for (pn = 0; pn < nmarks; pn++)
 	{
 	strcpy(p[4 * pn + 0].pict_type, "LINE");
@@ -1203,11 +1272,12 @@ dap_free(srtarg);
 dap_free(srtname);
 dap_free(plotvars);
 dap_free(plotmarks);
+dap_free(plotstyle);
 return p;
 }
 
-pict *plotlogreg(char *fname, char *yspec, char *x1list, int ngroups,
-					char *marks, int nmarks,  double level)
+pict *plotlogreg(char *fname, char *yspec, char *x1list, char *style,
+		int ngroups, char *marks, int nmarks,  double level)
 {
 int varv[3];
 char *trlname;
@@ -1228,6 +1298,7 @@ int c;
 int cs;
 pict *p;
 int pn;
+char *plotstyle;
 
 trlname = dap_malloc(strlen(fname) + 5, "");
 strcpy(trlname, fname);
@@ -1278,7 +1349,7 @@ while (step())
 	}
 
 strcpy(grparg, x1list);
-sprintf(grparg + strlen(grparg), " %d COUNT", ngroups);
+sprintf(grparg + strlen(grparg), " %d#", ngroups);
 
 group(trlname, grparg, marks);
 
@@ -1324,7 +1395,11 @@ strcat(plotvars, casevar);
 strcpy(plotmarks, marks);
 strcat(plotmarks, " _type_");
 
-p = plot(srtname, plotvars, plotmarks, "o4bb", NULL, NULL, 4 * nmarks);
+plotstyle = dap_malloc(strlen(style) + 4, "");
+strcpy(plotstyle, "o4 ");
+strcat(plotstyle, style);
+
+p = plot(srtname, plotvars, plotmarks, plotstyle, NULL, NULL, 4 * nmarks);
 for (pn = 0; pn < nmarks; pn ++)
 	{
 	strcpy(p[4 * pn + 0].pict_type, "LINE");
@@ -1345,6 +1420,7 @@ dap_free(srtname);
 dap_free(plotvars);
 dap_free(plotmarks);
 dap_free(casevar);
+dap_free(plotstyle);
 return p;
 }
 
