@@ -976,6 +976,8 @@ double k;
 double term;
 int sign;
 
+if (d == 0.0)
+	return 0.0;
 lambda = d * sqrt(n);
 lambda *= -2.0 * lambda;
 for (k = 1.0, l = 0.0, sign = 1;
