@@ -289,6 +289,11 @@ double probchisq(double c, int df)
 {
 double ddf;
 
+if (df < 0)
+	{
+	fprintf(dap_err, "(probchisq) non-positive df = %d\n", df);
+	exit(1);
+	}
 switch (df)
 	{
 case 1:
