@@ -61,7 +61,6 @@ char *tick_lab;			/* label */
 double tick_len;
 } tick;
 
-void pict_file(char fname[]);
 void pict_init(int orient, int bboxx0, int bboxy0, int bboxx1, int bboxy1, int npages);
 void pict_port(int npages);
 void pict_land(int npages);
@@ -90,3 +89,6 @@ double pict_autoaxes(pict *p, char xlab[], char ylab[], char axspec[],
 		double (*xfunct)(), double (*yfunct)(), char caption[], int autopos);
 void nport(pict *p, int nplots, int nperpage);
 void nland(pict *p, int nplots, int nperpage);
+
+void pict_save(pict *p, int npicts, char *dataset);
+pict *pict_rest(char *dataset);
